@@ -16,6 +16,7 @@ import ch
 
 class TestCh(unittest.TestCase):
     
+    
     def test_cachehits(self):
         """Test how many nodes are visited when cache is cleared. 
         If the number of hits changes, it has to be carefully
@@ -109,26 +110,40 @@ class TestCh(unittest.TestCase):
         self.assertTrue((dr0 - dr1).nnz == 0)
         
 
-    def test_sum_and_mean(self):
-        for fn in [ch.sum, ch.mean]:
-            data = ch.zeros((3,4,7,2))
-            dsum = fn(data, axis=2)
-            dr = dsum.dr_wrt(data)        
-            diff = ch.random.randn(data.size).reshape(data.shape)
-        
-            pred = dr.dot(diff.r.ravel())
-            gt = fn(diff, axis=2)
-            #print pred
-            #print gt
-            #print pred.ravel() - gt.r.ravel()
-            self.assertTrue(1e-15 > np.max(np.abs(gt.r.ravel() - pred)))
-        
-            # test caching
-            dr0 = gt.dr_wrt(diff)
-            diff[:] = np.random.randn(diff.size).reshape(diff.shape)
-            self.assertTrue(gt.dr_wrt(diff) is dr0) # changing values shouldn't force recompute
-            gt.axis=1
-            self.assertTrue(gt.dr_wrt(diff) is not dr0)
+    def test_sum_mean_std_var(self):
+        for fn in [ch.sum, ch.mean,ch.var, ch.std]:
+            
+            # Create fake input and differences in input space
+            data1 = ch.ones((3,4,7,2))
+            data2 = ch.array(data1.r + .1 * np.random.rand(data1.size).reshape(data1.shape))
+            diff = data2.r - data1.r
+
+            # Compute outputs
+            result1 = fn(data1, axis=2)
+            result2 = fn(data2, axis=2)
+
+            # Empirical and predicted derivatives
+            gt = result2.r - result1.r
+            pred = result1.dr_wrt(data1).dot(diff.ravel()).reshape(gt.shape)
+            
+            #print np.max(np.abs(gt - pred))
+            
+            if fn in [ch.std, ch.var]:
+                self.assertTrue(1e-2 > np.max(np.abs(gt - pred)))        
+            else:
+                self.assertTrue(1e-14 > np.max(np.abs(gt - pred)))        
+                # test caching
+                dr0 = result1.dr_wrt(data1)
+                data1[:] = np.random.randn(data1.size).reshape(data1.shape)
+                self.assertTrue(result1.dr_wrt(data1) is dr0) # changing values shouldn't force recompute
+                result1.axis=1
+                self.assertTrue(result1.dr_wrt(data1) is not dr0)
+            
+        self.assertEqual(ch.mean(ch.eye(3),axis=1).ndim, np.mean(np.eye(3),axis=1).ndim)
+        self.assertEqual(ch.mean(ch.eye(3),axis=0).ndim, np.mean(np.eye(3),axis=0).ndim)
+        self.assertEqual(ch.sum(ch.eye(3),axis=1).ndim, np.sum(np.eye(3),axis=1).ndim)
+        self.assertEqual(ch.sum(ch.eye(3),axis=0).ndim, np.sum(np.eye(3),axis=0).ndim)
+            
         
 
     def test_cumsum(self):
@@ -200,6 +215,15 @@ class TestCh(unittest.TestCase):
         
         self.assertFalse(np.any(residuals1))
         self.assertFalse(np.any(residuals2))
+        
+        d0 = ch.array(np.arange(60).reshape((10,6)))
+        d1 = ch.vstack((d0[:4], d0[4:]))
+        d2 = ch.hstack((d1[:,:3], d1[:,3:]))
+        tmp = d2.dr_wrt(d0).todense()
+        diff = tmp - np.eye(tmp.shape[0])
+        self.assertFalse(np.any(diff.ravel()))
+        
+        
 
     #def test_drs(self):
     #    a = ch.Ch(2)
